@@ -1,5 +1,5 @@
 /* GSequencer - Advanced GTK Sequencer
- * Copyright (C) 2005-2022 Joël Krähemann
+ * Copyright (C) 2005-2024 Joël Krähemann
  *
  * This file is part of GSequencer.
  *
@@ -22,7 +22,6 @@
 #include <ags/audio/ags_diatonic_scale.h>
 #include <ags/audio/ags_audio_signal.h>
 #include <ags/audio/ags_audio_buffer_util.h>
-#include <ags/audio/ags_sfz_synth_util.h>
 
 #include <ags/audio/file/ags_audio_container.h>
 #include <ags/audio/file/ags_audio_container_manager.h>
@@ -1619,7 +1618,8 @@ ags_sfz_synth_generator_compute(AgsSFZSynthGenerator *sfz_synth_generator,
   loop_start = 0;
   loop_end = 0;
 
-  audio_buffer_util_format = ags_audio_buffer_util_format_from_soundcard(format);
+  audio_buffer_util_format = ags_audio_buffer_util_format_from_soundcard(AGS_AUDIO_SIGNAL(audio_signal)->audio_buffer_util,
+									 format);
 
   buffer = ags_stream_alloc(frame_count,
 			    format);
@@ -1680,8 +1680,9 @@ ags_sfz_synth_generator_compute(AgsSFZSynthGenerator *sfz_synth_generator,
 
   g_rec_mutex_unlock(sfz_synth_generator_mutex);    
 
-  copy_mode = ags_audio_buffer_util_get_copy_mode(audio_buffer_util_format,
-						  audio_buffer_util_format);
+  copy_mode = ags_audio_buffer_util_get_copy_mode_from_format(AGS_AUDIO_SIGNAL(audio_signal)->audio_buffer_util,
+							      audio_buffer_util_format,
+							      audio_buffer_util_format);
   
   
   g_rec_mutex_lock(stream_mutex);
@@ -1695,7 +1696,8 @@ ags_sfz_synth_generator_compute(AgsSFZSynthGenerator *sfz_synth_generator,
       copy_count = frame_count - i;
     }
 
-    ags_audio_buffer_util_copy_buffer_to_buffer(stream->data, 1, 0,
+    ags_audio_buffer_util_copy_buffer_to_buffer(AGS_AUDIO_SIGNAL(audio_signal)->audio_buffer_util,
+						stream->data, 1, 0,
 						buffer, 1, i,
 						copy_count, copy_mode);
     i += copy_count;
