@@ -20,11 +20,18 @@
 #include <ags/audio/ags_volume_util.h>
 
 #include <ags/audio/ags_audio_buffer_util.h>
+#include <ags/audio/ags_vector_256_manager.h>
 
 #if defined(AGS_OSX_ACCELERATE_BUILTIN_FUNCTIONS)
+#include <ags/audio/ags_vdsp_256_manager.h>
+
 #include <Accelerate/Accelerate.h>
 
-#define LARGE_VECTOR 1
+#define AGS_VECTORIZED_BUILTIN_FUNCTIONS 1
+#define AGS_VECTOR_256_FUNCTIONS 1
+
+// #define LARGE_VECTOR 1
+// #define AGS_VDSP_256_FUNCTIONS
 #endif
 
 /**
@@ -73,16 +80,7 @@ ags_volume_util_alloc()
   ptr = (AgsVolumeUtil *) g_new(AgsVolumeUtil,
 				1);
 
-  ptr->destination = NULL;
-  ptr->destination_stride = 1;
-
-  ptr->source = NULL;
-  ptr->source_stride = 1;
-
-  ptr->buffer_length = 0;
-  ptr->format = AGS_VOLUME_UTIL_DEFAULT_FORMAT;
-
-  ptr->volume = 1.0;
+  ptr[0] = AGS_VOLUME_UTIL_INITIALIZER;
 
   return(ptr);
 }
@@ -101,6 +99,8 @@ gpointer
 ags_volume_util_copy(AgsVolumeUtil *ptr)
 {
   AgsVolumeUtil *new_ptr;
+
+  g_return_val_if_fail(ptr != NULL, NULL);
   
   new_ptr = (AgsVolumeUtil *) g_new(AgsVolumeUtil,
 				    1);
@@ -130,6 +130,8 @@ ags_volume_util_copy(AgsVolumeUtil *ptr)
 void
 ags_volume_util_free(AgsVolumeUtil *ptr)
 {
+  g_return_if_fail(ptr != NULL);
+
   g_free(ptr->destination);
 
   if(ptr->destination != ptr->source){
@@ -453,6 +455,40 @@ ags_volume_util_compute_s8(AgsVolumeUtil *volume_util)
 #if defined(AGS_VECTORIZED_BUILTIN_FUNCTIONS)
   i_stop = volume_util->buffer_length - (volume_util->buffer_length % 8);
 
+#if defined(AGS_VECTOR_256_FUNCTIONS)
+  AgsVector256Manager *vector_256_manager = ags_vector_256_manager_get_instance();
+  
+  while(i + 256 <= i_stop){
+    AgsVectorArr *vector_arr;
+
+    guint j;
+    
+    while((vector_arr = ags_vector_256_manager_try_acquire(vector_256_manager, AGS_VECTOR_256_DOUBLE)) == NULL){
+      g_thread_yield();
+    }
+
+    ags_audio_buffer_util_fill_v8double_from_s8(NULL,
+						 vector_arr->vector.vec_double->mem_double, 1,
+						 source, source_stride,
+						 32);
+
+    for(j = 0; j < 32; j++){
+      vector_arr->vector.vec_double->mem_double[j] *= volume_util->volume;
+    }
+    
+    ags_audio_buffer_util_fetch_v8double_as_s8(NULL,
+						source, source_stride,
+						vector_arr->vector.vec_double->mem_double, 1,
+						32);
+
+    ags_vector_256_manager_release(vector_256_manager,
+				   vector_arr);
+
+    source += (256 * source_stride);
+    i += 256;
+  }
+#endif
+
   for(; i < i_stop;){
     ags_v8double v_buffer;
 
@@ -515,6 +551,42 @@ ags_volume_util_compute_s8(AgsVolumeUtil *volume_util)
     
     i += ((volume_util->buffer_length < 256) ? volume_util->buffer_length: 256);
   }
+#elif defined(AGS_VDSP_256_FUNCTIONS)
+  AgsVDSP256Manager *vdsp_256_manager = ags_vdsp_256_manager_get_instance();
+
+  i_stop = volume_util->buffer_length - (volume_util->buffer_length % 8);
+  
+  while(i + 256 <= i_stop){
+    AgsVDSPArr *source_arr, *result_arr;
+
+    guint j;
+
+    while(!ags_vdsp_256_manager_try_acquire_dual(vdsp_256_manager,
+						 AGS_VDSP_256_DOUBLE, AGS_VDSP_256_DOUBLE,
+						 &source_arr, &result_arr)){
+      g_thread_yield();
+    }
+
+    ags_audio_buffer_util_put_double_from_s8(NULL,
+					     source_arr->vdsp.vec_double->mem_double, 1,
+					     source, source_stride,
+					     256);
+
+    vDSP_vsmulD(source_arr->vdsp.vec_double->mem_double, 1, &(volume_util->volume), result_arr->vdsp.vec_double->mem_double, 1, 256);
+    
+    ags_audio_buffer_util_get_double_as_s8(NULL,
+					   destination, destination_stride,
+					   result_arr->vdsp.vec_double->mem_double, 1,
+					   256);
+    
+    ags_vdsp_256_manager_release(vdsp_256_manager,
+				 source_arr);
+    ags_vdsp_256_manager_release(vdsp_256_manager,
+				 result_arr);
+
+    source += (256 * source_stride);
+    i += 256;
+  }
 #else
   i_stop = volume_util->buffer_length - (volume_util->buffer_length % 8);
   
@@ -613,6 +685,40 @@ ags_volume_util_compute_s16(AgsVolumeUtil *volume_util)
 #if defined(AGS_VECTORIZED_BUILTIN_FUNCTIONS)
   i_stop = volume_util->buffer_length - (volume_util->buffer_length % 8);
 
+#if defined(AGS_VECTOR_256_FUNCTIONS)
+  AgsVector256Manager *vector_256_manager = ags_vector_256_manager_get_instance();
+  
+  while(i + 256 <= i_stop){
+    AgsVectorArr *vector_arr;
+
+    guint j;
+    
+    while((vector_arr = ags_vector_256_manager_try_acquire(vector_256_manager, AGS_VECTOR_256_DOUBLE)) == NULL){
+      g_thread_yield();
+    }
+
+    ags_audio_buffer_util_fill_v8double_from_s16(NULL,
+						 vector_arr->vector.vec_double->mem_double, 1,
+						 source, source_stride,
+						 32);
+
+    for(j = 0; j < 32; j++){
+      vector_arr->vector.vec_double->mem_double[j] *= volume_util->volume;
+    }
+    
+    ags_audio_buffer_util_fetch_v8double_as_s16(NULL,
+						source, source_stride,
+						vector_arr->vector.vec_double->mem_double, 1,
+						32);
+
+    ags_vector_256_manager_release(vector_256_manager,
+				   vector_arr);
+
+    source += (256 * source_stride);
+    i += 256;
+  }
+#endif
+  
   for(; i < i_stop;){
     ags_v8double v_buffer;
 
@@ -675,6 +781,42 @@ ags_volume_util_compute_s16(AgsVolumeUtil *volume_util)
     
     i += ((volume_util->buffer_length < 256) ? volume_util->buffer_length: 256);
   }
+#elif defined(AGS_VDSP_256_FUNCTIONS)
+  AgsVDSP256Manager *vdsp_256_manager = ags_vdsp_256_manager_get_instance();
+
+  i_stop = volume_util->buffer_length - (volume_util->buffer_length % 8);
+  
+  while(i + 256 <= i_stop){
+    AgsVDSPArr *source_arr, *result_arr;
+
+    guint j;
+    
+    while(!ags_vdsp_256_manager_try_acquire_dual(vdsp_256_manager,
+						 AGS_VDSP_256_DOUBLE, AGS_VDSP_256_DOUBLE,
+						 &source_arr, &result_arr)){
+      g_thread_yield();
+    }
+
+    ags_audio_buffer_util_put_double_from_s16(NULL,
+					      source_arr->vdsp.vec_double->mem_double, 1,
+					     source, source_stride,
+					      256);
+
+    vDSP_vsmulD(source_arr->vdsp.vec_double->mem_double, 1, &(volume_util->volume), result_arr->vdsp.vec_double->mem_double, 1, 256);
+    
+    ags_audio_buffer_util_get_double_as_s16(NULL,
+					    destination, destination_stride,
+					    result_arr->vdsp.vec_double->mem_double, 1,
+					    256);
+    
+    ags_vdsp_256_manager_release(vdsp_256_manager,
+				 source_arr);
+    ags_vdsp_256_manager_release(vdsp_256_manager,
+				 result_arr);
+
+    source += (256 * source_stride);
+    i += 256;
+  }
 #else
   i_stop = volume_util->buffer_length - (volume_util->buffer_length % 8);
   
@@ -773,6 +915,40 @@ ags_volume_util_compute_s24(AgsVolumeUtil *volume_util)
 #if defined(AGS_VECTORIZED_BUILTIN_FUNCTIONS)
   i_stop = volume_util->buffer_length - (volume_util->buffer_length % 8);
 
+#if defined(AGS_VECTOR_256_FUNCTIONS)
+  AgsVector256Manager *vector_256_manager = ags_vector_256_manager_get_instance();
+  
+  while(i + 256 <= i_stop){
+    AgsVectorArr *vector_arr;
+
+    guint j;
+    
+    while((vector_arr = ags_vector_256_manager_try_acquire(vector_256_manager, AGS_VECTOR_256_DOUBLE)) == NULL){
+      g_thread_yield();
+    }
+
+    ags_audio_buffer_util_fill_v8double_from_s32(NULL,
+						 vector_arr->vector.vec_double->mem_double, 1,
+						 source, source_stride,
+						 32);
+
+    for(j = 0; j < 32; j++){
+      vector_arr->vector.vec_double->mem_double[j] *= volume_util->volume;
+    }
+    
+    ags_audio_buffer_util_fetch_v8double_as_s32(NULL,
+						source, source_stride,
+						vector_arr->vector.vec_double->mem_double, 1,
+						32);
+
+    ags_vector_256_manager_release(vector_256_manager,
+				   vector_arr);
+
+    source += (256 * source_stride);
+    i += 256;
+  }
+#endif
+
   for(; i < i_stop;){
     ags_v8double v_buffer;
 
@@ -835,6 +1011,42 @@ ags_volume_util_compute_s24(AgsVolumeUtil *volume_util)
     
     i += ((volume_util->buffer_length < 256) ? volume_util->buffer_length: 256);
   }
+#elif defined(AGS_VDSP_256_FUNCTIONS)
+  AgsVDSP256Manager *vdsp_256_manager = ags_vdsp_256_manager_get_instance();
+  
+  i_stop = volume_util->buffer_length - (volume_util->buffer_length % 8);
+  
+  while(i + 256 <= i_stop){
+    AgsVDSPArr *source_arr, *result_arr;
+
+    guint j;
+    
+    while(!ags_vdsp_256_manager_try_acquire_dual(vdsp_256_manager,
+						 AGS_VDSP_256_DOUBLE, AGS_VDSP_256_DOUBLE,
+						 &source_arr, &result_arr)){
+      g_thread_yield();
+    }
+
+    ags_audio_buffer_util_put_double_from_s32(NULL,
+					      source_arr->vdsp.vec_double->mem_double, 1,
+					     source, source_stride,
+					      256);
+
+    vDSP_vsmulD(source_arr->vdsp.vec_double->mem_double, 1, &(volume_util->volume), result_arr->vdsp.vec_double->mem_double, 1, 256);
+    
+    ags_audio_buffer_util_get_double_as_s32(NULL,
+					    destination, destination_stride,
+					    result_arr->vdsp.vec_double->mem_double, 1,
+					    256);
+
+    ags_vdsp_256_manager_release(vdsp_256_manager,
+				 source_arr);
+    ags_vdsp_256_manager_release(vdsp_256_manager,
+				 result_arr);
+
+    source += (256 * source_stride);
+    i += 256;
+  }
 #else
   i_stop = volume_util->buffer_length - (volume_util->buffer_length % 8);
   
@@ -933,6 +1145,40 @@ ags_volume_util_compute_s32(AgsVolumeUtil *volume_util)
 #if defined(AGS_VECTORIZED_BUILTIN_FUNCTIONS)
   i_stop = volume_util->buffer_length - (volume_util->buffer_length % 8);
 
+#if defined(AGS_VECTOR_256_FUNCTIONS)
+  AgsVector256Manager *vector_256_manager = ags_vector_256_manager_get_instance();
+  
+  while(i + 256 <= i_stop){
+    AgsVectorArr *vector_arr;
+
+    guint j;
+    
+    while((vector_arr = ags_vector_256_manager_try_acquire(vector_256_manager, AGS_VECTOR_256_DOUBLE)) == NULL){
+      g_thread_yield();
+    }
+
+    ags_audio_buffer_util_fill_v8double_from_s32(NULL,
+						 vector_arr->vector.vec_double->mem_double, 1,
+						 source, source_stride,
+						 32);
+
+    for(j = 0; j < 32; j++){
+      vector_arr->vector.vec_double->mem_double[j] *= volume_util->volume;
+    }
+    
+    ags_audio_buffer_util_fetch_v8double_as_s32(NULL,
+						source, source_stride,
+						vector_arr->vector.vec_double->mem_double, 1,
+						32);
+
+    ags_vector_256_manager_release(vector_256_manager,
+				   vector_arr);
+
+    source += (256 * source_stride);
+    i += 256;
+  }
+#endif
+
   for(; i < i_stop;){
     ags_v8double v_buffer;
 
@@ -995,6 +1241,42 @@ ags_volume_util_compute_s32(AgsVolumeUtil *volume_util)
     
     i += ((volume_util->buffer_length < 256) ? volume_util->buffer_length: 256);
   }
+#elif defined(AGS_VDSP_256_FUNCTIONS)
+  AgsVDSP256Manager *vdsp_256_manager = ags_vdsp_256_manager_get_instance();
+  
+  i_stop = volume_util->buffer_length - (volume_util->buffer_length % 8);
+  
+  while(i + 256 <= i_stop){
+    AgsVDSPArr *source_arr, *result_arr;
+
+    guint j;
+    
+    while(!ags_vdsp_256_manager_try_acquire_dual(vdsp_256_manager,
+						 AGS_VDSP_256_DOUBLE, AGS_VDSP_256_DOUBLE,
+						 &source_arr, &result_arr)){
+      g_thread_yield();
+    }
+
+    ags_audio_buffer_util_put_double_from_s32(NULL,
+					      source_arr->vdsp.vec_double->mem_double, 1,
+					      source, source_stride,
+					      256);
+
+    vDSP_vsmulD(source_arr->vdsp.vec_double->mem_double, 1, &(volume_util->volume), result_arr->vdsp.vec_double->mem_double, 1, 256);
+    
+    ags_audio_buffer_util_get_double_as_s32(NULL,
+					    destination, destination_stride,
+					    result_arr->vdsp.vec_double->mem_double, 1,
+					    256);
+    
+    ags_vdsp_256_manager_release(vdsp_256_manager,
+				 source_arr);
+    ags_vdsp_256_manager_release(vdsp_256_manager,
+				 result_arr);
+
+    source += (256 * source_stride);
+    i += 256;
+  }
 #else
   i_stop = volume_util->buffer_length - (volume_util->buffer_length % 8);
   
@@ -1093,6 +1375,42 @@ ags_volume_util_compute_s64(AgsVolumeUtil *volume_util)
 #if defined(AGS_VECTORIZED_BUILTIN_FUNCTIONS)
   i_stop = volume_util->buffer_length - (volume_util->buffer_length % 8);
 
+#if defined(AGS_VECTOR_256_FUNCTIONS)
+  AgsVector256Manager *vector_256_manager = ags_vector_256_manager_get_instance();
+
+  i_stop = volume_util->buffer_length - (volume_util->buffer_length % 8);
+  
+  while(i + 256 <= i_stop){
+    AgsVectorArr *vector_arr;
+
+    guint j;
+    
+    while((vector_arr = ags_vector_256_manager_try_acquire(vector_256_manager, AGS_VECTOR_256_DOUBLE)) == NULL){
+      g_thread_yield();
+    }
+
+    ags_audio_buffer_util_fill_v8double_from_s64(NULL,
+						 vector_arr->vector.vec_double->mem_double, 1,
+						 source, source_stride,
+						 32);
+
+    for(j = 0; j < 32; j++){
+      vector_arr->vector.vec_double->mem_double[j] *= volume_util->volume;
+    }
+    
+    ags_audio_buffer_util_fetch_v8double_as_s64(NULL,
+						source, source_stride,
+						vector_arr->vector.vec_double->mem_double, 1,
+						32);
+
+    ags_vector_256_manager_release(vector_256_manager,
+				   vector_arr);
+
+    source += (256 * source_stride);
+    i += 256;
+  }
+#endif
+
   for(; i < i_stop;){
     ags_v8double v_buffer;
 
@@ -1155,6 +1473,42 @@ ags_volume_util_compute_s64(AgsVolumeUtil *volume_util)
     
     i += ((volume_util->buffer_length < 256) ? volume_util->buffer_length: 256);
   }
+#elif defined(AGS_VDSP_256_FUNCTIONS)
+  AgsVDSP256Manager *vdsp_256_manager = ags_vdsp_256_manager_get_instance();
+
+  i_stop = volume_util->buffer_length - (volume_util->buffer_length % 8);
+  
+  while(i + 256 <= i_stop){
+    AgsVDSPArr *source_arr, *result_arr;
+
+    guint j;
+    
+    while(!ags_vdsp_256_manager_try_acquire_dual(vdsp_256_manager,
+						 AGS_VDSP_256_DOUBLE, AGS_VDSP_256_DOUBLE,
+						 &source_arr, &result_arr)){
+      g_thread_yield();
+    }
+
+    ags_audio_buffer_util_put_double_from_s64(NULL,
+					      source_arr->vdsp.vec_double->mem_double, 1,
+					      source, source_stride,
+					      256);
+
+    vDSP_vsmulD(source_arr->vdsp.vec_double->mem_double, 1, &(volume_util->volume), result_arr->vdsp.vec_double->mem_double, 1, 256);
+    
+    ags_audio_buffer_util_get_double_as_s64(NULL,
+					    destination, destination_stride,
+					    result_arr->vdsp.vec_double->mem_double, 1,
+					    256);
+    
+    ags_vdsp_256_manager_release(vdsp_256_manager,
+				 source_arr);
+    ags_vdsp_256_manager_release(vdsp_256_manager,
+				 result_arr);
+
+    source += (256 * source_stride);
+    i += 256;
+  }
 #else
   i_stop = volume_util->buffer_length - (volume_util->buffer_length % 8);
   
@@ -1253,6 +1607,40 @@ ags_volume_util_compute_float(AgsVolumeUtil *volume_util)
 #if defined(AGS_VECTORIZED_BUILTIN_FUNCTIONS)
   i_stop = volume_util->buffer_length - (volume_util->buffer_length % 8);
 
+#if defined(AGS_VECTOR_256_FUNCTIONS)
+  AgsVector256Manager *vector_256_manager = ags_vector_256_manager_get_instance();
+
+  while(i + 256 <= i_stop){
+    AgsVectorArr *vector_arr;
+
+    guint j;
+    
+    while((vector_arr = ags_vector_256_manager_try_acquire(vector_256_manager, AGS_VECTOR_256_DOUBLE)) == NULL){
+      g_thread_yield();
+    }
+
+    ags_audio_buffer_util_fill_v8double_from_float(NULL,
+						   vector_arr->vector.vec_double->mem_double, 1,
+						   source, source_stride,
+						   32);
+
+    for(j = 0; j < 32; j++){
+      vector_arr->vector.vec_double->mem_double[j] *= volume_util->volume;
+    }
+    
+    ags_audio_buffer_util_fetch_v8double_as_float(NULL,
+						  source, source_stride,
+						  vector_arr->vector.vec_double->mem_double, 1,
+						  32);
+
+    ags_vector_256_manager_release(vector_256_manager,
+				   vector_arr);
+
+    source += (256 * source_stride);
+    i += 256;
+  }
+#endif
+
   for(; i < i_stop;){
     ags_v8double v_buffer;
 
@@ -1315,6 +1703,42 @@ ags_volume_util_compute_float(AgsVolumeUtil *volume_util)
     
     i += ((volume_util->buffer_length < 256) ? volume_util->buffer_length: 256);
   }
+#elif defined(AGS_VDSP_256_FUNCTIONS)
+  AgsVDSP256Manager *vdsp_256_manager = ags_vdsp_256_manager_get_instance();
+  
+  i_stop = volume_util->buffer_length - (volume_util->buffer_length % 8);
+  
+  while(i + 256 <= i_stop){
+    AgsVDSPArr *source_arr, *result_arr;
+
+    guint j;
+    
+    while(!ags_vdsp_256_manager_try_acquire_dual(vdsp_256_manager,
+						 AGS_VDSP_256_DOUBLE, AGS_VDSP_256_DOUBLE,
+						 &source_arr, &result_arr)){
+      g_thread_yield();
+    }
+
+    ags_audio_buffer_util_put_double_from_float(NULL,
+						source_arr->vdsp.vec_double->mem_double, 1,
+						source, source_stride,
+						256);
+
+    vDSP_vsmulD(source_arr->vdsp.vec_double->mem_double, 1, &(volume_util->volume), result_arr->vdsp.vec_double->mem_double, 1, 256);
+    
+    ags_audio_buffer_util_get_double_as_float(NULL,
+					      destination, destination_stride,
+					      result_arr->vdsp.vec_double->mem_double, 1,
+					      256);
+
+    ags_vdsp_256_manager_release(vdsp_256_manager,
+				 source_arr);
+    ags_vdsp_256_manager_release(vdsp_256_manager,
+				 result_arr);
+
+    source += (256 * source_stride);
+    i += 256;
+  }
 #else
   i_stop = volume_util->buffer_length - (volume_util->buffer_length % 8);
   
@@ -1413,6 +1837,40 @@ ags_volume_util_compute_double(AgsVolumeUtil *volume_util)
 #if defined(AGS_VECTORIZED_BUILTIN_FUNCTIONS)
   i_stop = volume_util->buffer_length - (volume_util->buffer_length % 8);
 
+#if defined(AGS_VECTOR_256_FUNCTIONS)
+  AgsVector256Manager *vector_256_manager = ags_vector_256_manager_get_instance();
+  
+  while(i + 256 <= i_stop){
+    AgsVectorArr *vector_arr;
+
+    guint j;
+    
+    while((vector_arr = ags_vector_256_manager_try_acquire(vector_256_manager, AGS_VECTOR_256_DOUBLE)) == NULL){
+      g_thread_yield();
+    }
+
+    ags_audio_buffer_util_fill_v8double(NULL,
+					vector_arr->vector.vec_double->mem_double, 1,
+					source, source_stride,
+					32);
+    
+    for(j = 0; j < 32; j++){
+      vector_arr->vector.vec_double->mem_double[j] *= volume_util->volume;
+    }
+    
+    ags_audio_buffer_util_fetch_v8double(NULL,
+					 source, source_stride,
+					 vector_arr->vector.vec_double->mem_double, 1,
+					 32);
+
+    ags_vector_256_manager_release(vector_256_manager,
+				   vector_arr);
+
+    source += (256 * source_stride);
+    i += 256;
+  }
+#endif
+
   for(; i < i_stop;){
     ags_v8double v_buffer;
 
@@ -1475,6 +1933,42 @@ ags_volume_util_compute_double(AgsVolumeUtil *volume_util)
     
     i += ((volume_util->buffer_length < 256) ? volume_util->buffer_length: 256);
   }
+#elif defined(AGS_VDSP_256_FUNCTIONS)
+  AgsVDSP256Manager *vdsp_256_manager = ags_vdsp_256_manager_get_instance();
+  
+  i_stop = volume_util->buffer_length - (volume_util->buffer_length % 8);
+  
+  while(i + 256 <= i_stop){
+    AgsVDSPArr *source_arr, *result_arr;
+
+    guint j;
+    
+    while(!ags_vdsp_256_manager_try_acquire_dual(vdsp_256_manager,
+						 AGS_VDSP_256_DOUBLE, AGS_VDSP_256_DOUBLE,
+						 &source_arr, &result_arr)){
+      g_thread_yield();
+    }
+
+    ags_audio_buffer_util_put_double(NULL,
+				     source_arr->vdsp.vec_double->mem_double, 1,
+				     source, source_stride,
+				     256);
+    
+    vDSP_vsmulD(source_arr->vdsp.vec_double->mem_double, 1, &(volume_util->volume), result_arr->vdsp.vec_double->mem_double, 1, 256);
+    
+    ags_audio_buffer_util_put_double(NULL,
+				     destination, destination_stride,
+				     result_arr->vdsp.vec_double->mem_double, 1,
+				     256);
+    
+    ags_vdsp_256_manager_release(vdsp_256_manager,
+				 source_arr);
+    ags_vdsp_256_manager_release(vdsp_256_manager,
+				 result_arr);
+
+    source += (256 * source_stride);
+    i += 256;
+  }
 #else
   i_stop = volume_util->buffer_length - (volume_util->buffer_length % 8);
   
