@@ -1,5 +1,5 @@
 /* GSequencer - Advanced GTK Sequencer
- * Copyright (C) 2005-2023 Joël Krähemann
+ * Copyright (C) 2005-2024 Joël Krähemann
  *
  * This file is part of GSequencer.
  *
@@ -20,7 +20,6 @@
 #include <ags/audio/fx/ags_fx_fm_synth_audio_signal.h>
 
 #include <ags/audio/ags_port.h>
-#include <ags/audio/ags_audio_buffer_util.h>
 
 #include <ags/audio/fx/ags_fx_fm_synth_audio.h>
 #include <ags/audio/fx/ags_fx_fm_synth_audio_processor.h>
@@ -194,6 +193,8 @@ ags_fx_fm_synth_audio_signal_stream_feed(AgsFxNotationAudioSignal *fx_notation_a
   GRecMutex *fx_fm_synth_audio_mutex;
   GRecMutex *fx_fm_synth_audio_processor_mutex;
 
+  fx_fm_synth_audio_signal = (AgsFxFMSynthAudioSignal *) fx_notation_audio_signal;
+  
   audio = NULL;
   
   fx_fm_synth_audio = NULL;
@@ -255,10 +256,12 @@ ags_fx_fm_synth_audio_signal_stream_feed(AgsFxNotationAudioSignal *fx_notation_a
 	       "x1-256th", &x1_256th,
 	       NULL);
 
-  audio_buffer_util_format = ags_audio_buffer_util_format_from_soundcard(format);
+  audio_buffer_util_format = ags_audio_buffer_util_format_from_soundcard(&(fx_fm_synth_audio_signal->audio_buffer_util),
+									 format);
 
-  copy_mode = ags_audio_buffer_util_get_copy_mode(audio_buffer_util_format,
-						  audio_buffer_util_format);
+  copy_mode = ags_audio_buffer_util_get_copy_mode_from_format(&(fx_fm_synth_audio_signal->audio_buffer_util),
+							      audio_buffer_util_format,
+							      audio_buffer_util_format);
   
   /* get synth mutex */
   fx_fm_synth_audio_mutex = AGS_RECALL_GET_OBJ_MUTEX(fx_fm_synth_audio);
@@ -1146,8 +1149,10 @@ ags_fx_fm_synth_audio_signal_stream_feed(AgsFxNotationAudioSignal *fx_notation_a
     }
   }
 
-  copy_mode_out = ags_audio_buffer_util_get_copy_mode(ags_audio_buffer_util_format_from_soundcard(format),
-						      AGS_AUDIO_BUFFER_UTIL_FLOAT);
+  copy_mode_out = ags_audio_buffer_util_get_copy_mode_from_format(&(fx_fm_synth_audio_signal->audio_buffer_util),
+								  ags_audio_buffer_util_format_from_soundcard(&(fx_fm_synth_audio_signal->audio_buffer_util),
+													      format),
+								  AGS_AUDIO_BUFFER_UTIL_FLOAT);
 
   if(midi_note >= 0 &&
      midi_note < 128){
@@ -1523,10 +1528,12 @@ ags_fx_fm_synth_audio_signal_stream_feed(AgsFxNotationAudioSignal *fx_notation_a
 
       g_rec_mutex_unlock(source_stream_mutex);
 
-      ags_audio_buffer_util_clear_buffer(source->stream_current->data, 1,
+      ags_audio_buffer_util_clear_buffer(&(fx_fm_synth_audio_signal->audio_buffer_util),
+					 source->stream_current->data, 1,
 					 buffer_size, audio_buffer_util_format);
 
-      ags_audio_buffer_util_copy_buffer_to_buffer(source->stream_current->data, 1, 0,
+      ags_audio_buffer_util_copy_buffer_to_buffer(&(fx_fm_synth_audio_signal->audio_buffer_util),
+						  source->stream_current->data, 1, 0,
 						  ags_chorus_util_get_destination(channel_data->chorus_util), 1, 0,
 						  buffer_size, copy_mode);
 
