@@ -52,6 +52,10 @@ G_BEGIN_DECLS
 #define AGS_SOUNDCARD_MIN_BUFFER_SIZE (16)
 #define AGS_SOUNDCARD_MAX_BUFFER_SIZE (16384)
 
+#define AGS_SOUNDCARD_DEFAULT_USE_CACHE (FALSE)
+
+#define AGS_SOUNDCARD_DEFAULT_CACHE_BUFFER_SIZE (4096)
+
 #define AGS_SOUNDCARD_DEFAULT_BPM (120.0)
 #define AGS_SOUNDCARD_DEFAULT_DELAY_FACTOR (1.0 / 4.0)
 #define AGS_SOUNDCARD_DEFAULT_JIFFIE ((double) AGS_SOUNDCARD_DEFAULT_SAMPLERATE / (double) AGS_SOUNDCARD_DEFAULT_BUFFER_SIZE)
