@@ -389,7 +389,7 @@ ags_vst3_manager_get_filenames(AgsVst3Manager *vst3_manager)
       i++;
     }else{
 #ifdef HAVE_GLIB_2_44
-      contains_filename = g_strv_contains(filenames,
+      contains_filename = g_strv_contains((const gchar * const *) filenames,
 					  filename);
 #else
       contains_filename = ags_strv_contains(filenames,
